(window["webpackJsonp"] = window["webpackJsonp"] || []).push([[4],{

/***/ "./node_modules/cache-loader/dist/cjs.js?!./node_modules/babel-loader/lib/index.js!./node_modules/cache-loader/dist/cjs.js?!./node_modules/vue-loader/lib/index.js?!./src/views/metadata/Model.vue?vue&type=script&lang=js&":
/*!******************************************************************************************************************************************************************************************************************************************************!*\
  !*** ./node_modules/cache-loader/dist/cjs.js??ref--12-0!./node_modules/babel-loader/lib!./node_modules/cache-loader/dist/cjs.js??ref--0-0!./node_modules/vue-loader/lib??vue-loader-options!./src/views/metadata/Model.vue?vue&type=script&lang=js& ***!
  \******************************************************************************************************************************************************************************************************************************************************/
/*! exports provided: default */
/***/ (function(module, __webpack_exports__, __webpack_require__) {

"use strict";
eval("__webpack_require__.r(__webpack_exports__);\n/* harmony import */ var moment__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! moment */ \"./node_modules/moment/moment.js\");\n/* harmony import */ var moment__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(moment__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _components__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @/components */ \"./src/components/index.js\");\n/* harmony import */ var _api_manage__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @/api/manage */ \"./src/api/manage.js\");\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n//\n\n\n\nvar columns = [{\n  title: 'Model Name',\n  dataIndex: 'model_name',\n  sorter: true\n}, {\n  title: 'Model Desc',\n  dataIndex: 'model_desc'\n}];\n/* harmony default export */ __webpack_exports__[\"default\"] = ({\n  name: 'Model',\n  components: {\n    STable: _components__WEBPACK_IMPORTED_MODULE_1__[\"STable\"],\n    Ellipsis: _components__WEBPACK_IMPORTED_MODULE_1__[\"Ellipsis\"]\n  },\n  data: function data() {\n    var _this = this;\n\n    this.columns = columns;\n    return {\n      confirmLoading: false,\n      advanced: false,\n      queryParam: {},\n      loadData: function loadData(parameter) {\n        var requestParameters = Object.assign({}, parameter, _this.queryParam);\n        console.log('loadData request parameters:', requestParameters);\n        return Object(_api_manage__WEBPACK_IMPORTED_MODULE_2__[\"getModels\"])(requestParameters).then(function (res) {\n          console.log(res);\n          return res;\n        });\n      },\n      version: ''\n    };\n  },\n  mounted: function mounted() {\n    this.getAIFlowVersion();\n  },\n  methods: {\n    resetSearchForm: function resetSearchForm() {\n      this.queryParam = {\n        date: moment__WEBPACK_IMPORTED_MODULE_0___default()(new Date())\n      };\n    },\n    getAIFlowVersion: function getAIFlowVersion() {\n      var _this2 = this;\n\n      Object(_api_manage__WEBPACK_IMPORTED_MODULE_2__[\"getVersion\"])().then(function (res) {\n        _this2.version = res;\n      });\n    }\n  }\n});\n\n//# sourceURL=webpack:///./src/views/metadata/Model.vue?./node_modules/cache-loader/dist/cjs.js??ref--12-0!./node_modules/babel-loader/lib!./node_modules/cache-loader/dist/cjs.js??ref--0-0!./node_modules/vue-loader/lib??vue-loader-options");

/***/ }),

/***/ "./node_modules/cache-loader/dist/cjs.js?{\"cacheDirectory\":\"node_modules/.cache/vue-loader\",\"cacheIdentifier\":\"953cd07e-vue-loader-template\"}!./node_modules/vue-loader/lib/loaders/templateLoader.js?!./node_modules/cache-loader/dist/cjs.js?!./node_modules/vue-loader/lib/index.js?!./src/views/metadata/Model.vue?vue&type=template&id=bdc0777a&":
/*!**************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************!*\
  !*** ./node_modules/cache-loader/dist/cjs.js?{"cacheDirectory":"node_modules/.cache/vue-loader","cacheIdentifier":"953cd07e-vue-loader-template"}!./node_modules/vue-loader/lib/loaders/templateLoader.js??vue-loader-options!./node_modules/cache-loader/dist/cjs.js??ref--0-0!./node_modules/vue-loader/lib??vue-loader-options!./src/views/metadata/Model.vue?vue&type=template&id=bdc0777a& ***!
  \**************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/
/*! exports provided: render, staticRenderFns */
/***/ (function(module, __webpack_exports__, __webpack_require__) {

"use strict";
eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export (binding) */ __webpack_require__.d(__webpack_exports__, \"render\", function() { return render; });\n/* harmony export (binding) */ __webpack_require__.d(__webpack_exports__, \"staticRenderFns\", function() { return staticRenderFns; });\nvar render = function() {\n  var this$1 = this\n  var _vm = this\n  var _h = _vm.$createElement\n  var _c = _vm._self._c || _h\n  return _c(\n    \"page-header-wrapper\",\n    [\n      _c(\n        \"a-card\",\n        { attrs: { bordered: false } },\n        [\n          _c(\n            \"div\",\n            { staticClass: \"table-page-search-wrapper\" },\n            [\n              _c(\n                \"a-form\",\n                { attrs: { layout: \"inline\" } },\n                [\n                  _c(\n                    \"a-row\",\n                    { attrs: { gutter: 48 } },\n                    [\n                      _c(\n                        \"a-col\",\n                        { attrs: { md: 8, sm: 24 } },\n                        [\n                          _c(\n                            \"a-form-item\",\n                            { attrs: { label: \"Name\" } },\n                            [\n                              _c(\"a-input\", {\n                                attrs: { placeholder: \"\" },\n                                model: {\n                                  value: _vm.queryParam.model_name,\n                                  callback: function($$v) {\n                                    _vm.$set(_vm.queryParam, \"model_name\", $$v)\n                                  },\n                                  expression: \"queryParam.model_name\"\n                                }\n                              })\n                            ],\n                            1\n                          )\n                        ],\n                        1\n                      ),\n                      _c(\n                        \"a-col\",\n                        { attrs: { md: (!_vm.advanced && 8) || 24, sm: 24 } },\n                        [\n                          _c(\n                            \"span\",\n                            {\n                              staticClass: \"table-page-search-submitButtons\",\n                              style:\n                                (_vm.advanced && {\n                                  float: \"right\",\n                                  overflow: \"hidden\"\n                                }) ||\n                                {}\n                            },\n                            [\n                              _c(\n                                \"a-button\",\n                                {\n                                  attrs: { type: \"primary\" },\n                                  on: {\n                                    click: function($event) {\n                                      return _vm.$refs.table.refresh(true)\n                                    }\n                                  }\n                                },\n                                [_vm._v(\"Query\")]\n                              ),\n                              _c(\n                                \"a-button\",\n                                {\n                                  staticStyle: { \"margin-left\": \"8px\" },\n                                  on: {\n                                    click: function() {\n                                      return (this$1.queryParam = {})\n                                    }\n                                  }\n                                },\n                                [_vm._v(\"Reset\")]\n                              )\n                            ],\n                            1\n                          )\n                        ]\n                      )\n                    ],\n                    1\n                  )\n                ],\n                1\n              )\n            ],\n            1\n          ),\n          _c(\"s-table\", {\n            ref: \"table\",\n            attrs: {\n              size: \"default\",\n              rowKey: \"key\",\n              columns: _vm.columns,\n              data: _vm.loadData,\n              showPagination: \"auto\"\n            }\n          })\n        ],\n        1\n      ),\n      _c(\"a-card\", { attrs: { bordered: false } }, [\n        _c(\"span\", [\n          _vm._v(\"Version: \"),\n          _c(\n            \"a\",\n            {\n              attrs: {\n                href: \"https://pypi.org/project/ai-flow/\" + _vm.version,\n                target: \"_blank\"\n              }\n            },\n            [_vm._v(_vm._s(_vm.version))]\n          )\n        ])\n      ])\n    ],\n    1\n  )\n}\nvar staticRenderFns = []\nrender._withStripped = true\n\n\n\n//# sourceURL=webpack:///./src/views/metadata/Model.vue?./node_modules/cache-loader/dist/cjs.js?%7B%22cacheDirectory%22:%22node_modules/.cache/vue-loader%22,%22cacheIdentifier%22:%22953cd07e-vue-loader-template%22%7D!./node_modules/vue-loader/lib/loaders/templateLoader.js??vue-loader-options!./node_modules/cache-loader/dist/cjs.js??ref--0-0!./node_modules/vue-loader/lib??vue-loader-options");

/***/ }),

/***/ "./src/views/metadata/Model.vue":
/*!**************************************!*\
  !*** ./src/views/metadata/Model.vue ***!
  \**************************************/
/*! exports provided: default */
/***/ (function(module, __webpack_exports__, __webpack_require__) {

"use strict";
eval("__webpack_require__.r(__webpack_exports__);\n/* harmony import */ var _Model_vue_vue_type_template_id_bdc0777a___WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Model.vue?vue&type=template&id=bdc0777a& */ \"./src/views/metadata/Model.vue?vue&type=template&id=bdc0777a&\");\n/* harmony import */ var _Model_vue_vue_type_script_lang_js___WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Model.vue?vue&type=script&lang=js& */ \"./src/views/metadata/Model.vue?vue&type=script&lang=js&\");\n/* empty/unused harmony star reexport *//* harmony import */ var _node_modules_vue_loader_lib_runtime_componentNormalizer_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../../node_modules/vue-loader/lib/runtime/componentNormalizer.js */ \"./node_modules/vue-loader/lib/runtime/componentNormalizer.js\");\n\n\n\n\n\n/* normalize component */\n\nvar component = Object(_node_modules_vue_loader_lib_runtime_componentNormalizer_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(\n  _Model_vue_vue_type_script_lang_js___WEBPACK_IMPORTED_MODULE_1__[\"default\"],\n  _Model_vue_vue_type_template_id_bdc0777a___WEBPACK_IMPORTED_MODULE_0__[\"render\"],\n  _Model_vue_vue_type_template_id_bdc0777a___WEBPACK_IMPORTED_MODULE_0__[\"staticRenderFns\"],\n  false,\n  null,\n  null,\n  null\n  \n)\n\n/* hot reload */\nif (false) { var api; }\ncomponent.options.__file = \"src/views/metadata/Model.vue\"\n/* harmony default export */ __webpack_exports__[\"default\"] = (component.exports);\n\n//# sourceURL=webpack:///./src/views/metadata/Model.vue?");

/***/ }),

/***/ "./src/views/metadata/Model.vue?vue&type=script&lang=js&":
/*!***************************************************************!*\
  !*** ./src/views/metadata/Model.vue?vue&type=script&lang=js& ***!
  \***************************************************************/
/*! exports provided: default */
/***/ (function(module, __webpack_exports__, __webpack_require__) {

"use strict";
eval("__webpack_require__.r(__webpack_exports__);\n/* harmony import */ var _node_modules_cache_loader_dist_cjs_js_ref_12_0_node_modules_babel_loader_lib_index_js_node_modules_cache_loader_dist_cjs_js_ref_0_0_node_modules_vue_loader_lib_index_js_vue_loader_options_Model_vue_vue_type_script_lang_js___WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! -!../../../node_modules/cache-loader/dist/cjs.js??ref--12-0!../../../node_modules/babel-loader/lib!../../../node_modules/cache-loader/dist/cjs.js??ref--0-0!../../../node_modules/vue-loader/lib??vue-loader-options!./Model.vue?vue&type=script&lang=js& */ \"./node_modules/cache-loader/dist/cjs.js?!./node_modules/babel-loader/lib/index.js!./node_modules/cache-loader/dist/cjs.js?!./node_modules/vue-loader/lib/index.js?!./src/views/metadata/Model.vue?vue&type=script&lang=js&\");\n/* empty/unused harmony star reexport */ /* harmony default export */ __webpack_exports__[\"default\"] = (_node_modules_cache_loader_dist_cjs_js_ref_12_0_node_modules_babel_loader_lib_index_js_node_modules_cache_loader_dist_cjs_js_ref_0_0_node_modules_vue_loader_lib_index_js_vue_loader_options_Model_vue_vue_type_script_lang_js___WEBPACK_IMPORTED_MODULE_0__[\"default\"]); \n\n//# sourceURL=webpack:///./src/views/metadata/Model.vue?");

/***/ }),

/***/ "./src/views/metadata/Model.vue?vue&type=template&id=bdc0777a&":
/*!*********************************************************************!*\
  !*** ./src/views/metadata/Model.vue?vue&type=template&id=bdc0777a& ***!
  \*********************************************************************/
/*! exports provided: render, staticRenderFns */
/***/ (function(module, __webpack_exports__, __webpack_require__) {

"use strict";
eval("__webpack_require__.r(__webpack_exports__);\n/* harmony import */ var _node_modules_cache_loader_dist_cjs_js_cacheDirectory_node_modules_cache_vue_loader_cacheIdentifier_953cd07e_vue_loader_template_node_modules_vue_loader_lib_loaders_templateLoader_js_vue_loader_options_node_modules_cache_loader_dist_cjs_js_ref_0_0_node_modules_vue_loader_lib_index_js_vue_loader_options_Model_vue_vue_type_template_id_bdc0777a___WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! -!../../../node_modules/cache-loader/dist/cjs.js?{\"cacheDirectory\":\"node_modules/.cache/vue-loader\",\"cacheIdentifier\":\"953cd07e-vue-loader-template\"}!../../../node_modules/vue-loader/lib/loaders/templateLoader.js??vue-loader-options!../../../node_modules/cache-loader/dist/cjs.js??ref--0-0!../../../node_modules/vue-loader/lib??vue-loader-options!./Model.vue?vue&type=template&id=bdc0777a& */ \"./node_modules/cache-loader/dist/cjs.js?{\\\"cacheDirectory\\\":\\\"node_modules/.cache/vue-loader\\\",\\\"cacheIdentifier\\\":\\\"953cd07e-vue-loader-template\\\"}!./node_modules/vue-loader/lib/loaders/templateLoader.js?!./node_modules/cache-loader/dist/cjs.js?!./node_modules/vue-loader/lib/index.js?!./src/views/metadata/Model.vue?vue&type=template&id=bdc0777a&\");\n/* harmony reexport (safe) */ __webpack_require__.d(__webpack_exports__, \"render\", function() { return _node_modules_cache_loader_dist_cjs_js_cacheDirectory_node_modules_cache_vue_loader_cacheIdentifier_953cd07e_vue_loader_template_node_modules_vue_loader_lib_loaders_templateLoader_js_vue_loader_options_node_modules_cache_loader_dist_cjs_js_ref_0_0_node_modules_vue_loader_lib_index_js_vue_loader_options_Model_vue_vue_type_template_id_bdc0777a___WEBPACK_IMPORTED_MODULE_0__[\"render\"]; });\n\n/* harmony reexport (safe) */ __webpack_require__.d(__webpack_exports__, \"staticRenderFns\", function() { return _node_modules_cache_loader_dist_cjs_js_cacheDirectory_node_modules_cache_vue_loader_cacheIdentifier_953cd07e_vue_loader_template_node_modules_vue_loader_lib_loaders_templateLoader_js_vue_loader_options_node_modules_cache_loader_dist_cjs_js_ref_0_0_node_modules_vue_loader_lib_index_js_vue_loader_options_Model_vue_vue_type_template_id_bdc0777a___WEBPACK_IMPORTED_MODULE_0__[\"staticRenderFns\"]; });\n\n\n\n//# sourceURL=webpack:///./src/views/metadata/Model.vue?");

/***/ })

}]);