#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
from typing import Text
from kubernetes import config

kubernetes_cluster_available = False


def load_kubernetes_config(config_file_path: Text = None):
    global kubernetes_cluster_available
    if config_file_path is not None:
        # load kubernetes config from user provided config file
        config.load_kube_config(config_file=config_file_path)
        kubernetes_cluster_available = True
    else:
        try:
            # load local kubernetes config
            config.load_kube_config()
            kubernetes_cluster_available = True
        except Exception as e:
            try:
                config.load_incluster_config()
                kubernetes_cluster_available = True
            except Exception as e:
                pass
