#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""task_instance_add_is_schedulable

Revision ID: 818f168686ac
Revises: cf3cb8473f3c
Create Date: 2021-12-28 15:26:13.728326

"""

import sqlalchemy as sa
from alembic import op


# revision identifiers, used by Alembic.
revision = '818f168686ac'
down_revision = 'cf3cb8473f3c'
branch_labels = None
depends_on = None


def upgrade():
    """Apply task_instance_add_is_schedulable"""
    op.add_column('task_instance', sa.Column('is_schedulable', sa.Boolean, nullable=True, default=True))


def downgrade():
    """Unapply task_instance_add_is_schedulable"""
    op.drop_column('task_instance', 'is_schedulable')
